#!/usr/bin/python
# A Python client for HTTPSpeaker.
__id__ = "$Id$"
__author__ = "Jason White"
__version__ = "$Revision$"
__date__ = "$Date$"
__copyright__ = "Copyright 2011 Jason White"
__license__ = "LGPL"

from httplib import HTTPConnection, HTTPResponse

class HTTPSpeakerError(Exception):
    "Holds the HTTP status code and reason phrase returned by the server in the event of an error."
    def __init__(self, status, reason):
        self.status = status
        self.reason = reason
    def __str__(self):
        return "Status code " + str(self.status) + ": " + self.reason

class HTTPSpeakerClient:
    """Emacspeak HTTP speech client, for HTTPSpeaker instances."""
    def __init__(self, host="127.0.0.1", port=8000):
        "Initialize client to connect to server at given host and port."
        self._connection=HTTPConnection(host, port)

    def postCommand(self, command, arg=""):
        """Post command, with argument arg (default, empty), to the speech server.
        Returns the body of the server's HTTP response, if any. On error, HTTPSpeakerError is raised."""
        body = command
        if arg:
            body += ": " + arg
        self._connection.request("POST", "/", body, {"Content-type": "text/plain"})
        response=self._connection.getresponse()
        if response.status != 200:
            raise HTTPSpeakerError(response.status, response.reason)
        return response.read()

    def speak(self, text):
        "Speak the supplied string."
        self.postCommand("speak", text)

    def stop(self):
        "Stop speaking."
        self.postCommand("stop")

    def isSpeaking(self):
        "Return '0' when not speaking."
        return self.postCommand("isSpeaking")

    def close(self):
        "Close the connection to the speech server."
        self._connection.close()

# Test the server (assumes localhost:8000).
if __name__ == '__main__':
    client = HTTPSpeakerClient()
    client.speak("Hello, world!")
    print client.isSpeaking()
    client.close()



#!/usr/bin/python

"""HTTP wrapper around Emacspeak speech server.

Speech server is launched on HTTP server startup.

Speech commands are invoked via URLs:

http://host:port/cmd?arg

calls speaker.cmd(arg)

"""

__id__ = "$Id: HTTPSpeaker.py 7067 2011-06-25 02:49:51Z tv.raman.tv $"
__author__ = "$Author: tv.raman.tv $"
__version__ = "$Revision: 7067 $"
__date__ = "$Date: 2011-06-24 19:49:51 -0700 (Fri, 24 Jun 2011) $"
__copyright__ = "Copyright (c) 2005 T. V. Raman"
__license__ = "LGPL"


from speaker import Speaker
from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
import sys
import os
import urllib

class HTTPSpeaker (HTTPServer):

    """Speech server via HTTP."""

    def __init__(self, address, handler,
                 engine='outloud',
                 rate = 75):
        """Initialize HTTP listener."""
        HTTPServer.__init__(self, address, handler)
        self.speaker = Speaker(engine,
                               'localhost',
                               {'punctuations' : 'some',
                                'rate' : rate})

class SpeakHTTPRequestHandler(BaseHTTPRequestHandler):

    """Handle HTTP Speak requests."""
    handlers = ['say',
                'speak',
                'letter',
                'addText',
                'silence',
                'tone',
                'stop',
                'punctuation',
                'rate',
                'allcaps',
                'capitalize',
                'splitcaps',
                'reset',
                'shutdown',
                'version'             ]

    def do_GET(self):
        """Not Implemented."""
        #cmd = None
        #arg = None
        if hasattr(self.server.speaker, cmd):
            method = getattr(self.server.speaker, cmd)
            if arg is None:
                method()
            else:
                method(urllib.unquote(arg))
            self.send_response(200, self.path)
        else: self.send_error(501, "Speaker error")

    def do_POST(self):
        """Handle speech request in a POST message. """
        contentLength = self.headers.getheader('content-length')
        if contentLength:
            contentLength = int(contentLength)
            inputBody = self.rfile.read(contentLength)
            sys.stderr.write(inputBody  + "\n")
            if inputBody.startswith("speak:"):
                text = inputBody[6:]
                self.server.speaker.sayUtterances([text] )
                self.send_response(200, 'OK')
            elif inputBody == "stop":
                self.server.speaker.stop()
                self.send_response(200, 'OK')
            elif inputBody == "isSpeaking":
                self.send_response(200, 'OK')
                self.send_header("Content-type", "text/html")
                self.end_headers()
                self.wfile.write("0")
            else:
                self.send_error(501, 'Unknown POST message ' + inputBody)
    
def start():
    if sys.argv[1:]:
        engine = sys.argv[1]
    else:
        engine=os.getenv('DTK_PROGRAM','outloud')
    if sys.argv[2:]:
        port = int(sys.argv[2])
    else:
        port = 8000
    if sys.argv[3:]:
        rate = int(sys.argv[3])
    else:
        rate = 75
    server_address = ('', port)
    httpd = HTTPSpeaker  (server_address,
    SpeakHTTPRequestHandler, engine, rate)
    httpd.serve_forever()


if __name__ == '__main__':
    start()



#!/usr/bin/python
#
# Copyright (C) 2008 Google Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Provide a Contacts interface for calling from Emacs
# Cloned from GData library sample

__author__ = 'T. V. Raman'

import getpass
import atom
import gdata.contacts
import gdata.contacts.service
# fix for now:
gdata.contacts.REL_MOBILE='http://schemas.google.com/g/2005#mobile'
class ContactsShell(object):
  """ContactsShell provides Emacs-g-client access to Contact lists.""""ContactsSample object demonstrates operations with the Contacts feed."""

  def __init__(self, email, password):
    """Constructor for the ContactsShell object.

    Takes an email and password corresponding to a gmail account 
    and sets up things for Emacs to access Contact services.

    Args       :
      email    : [string] The e-mail address of the account to use for the sample.
      password : [string] The password 
    Yields     : A ContactsShell handle.

    """
    self.gd_client = gdata.contacts.service.ContactsService()
    self.gd_client.email = email
    self.gd_client.password = password
    self.gd_client.source = 'Emacs G-Client Contacts Shell'
    self.gd_client.ProgrammaticLogin()

  def PrintFeed(self, feed):
    """Print out feed so Emacs can use it.

    Args   :
      feed : A gdata.contacts.ContactsFeed instance.
    """
    print '\n'
    if not feed.entry:
      print 'No entries in feed.\n'
    for i, entry in enumerate(feed.entry):
      print '\n%s %s' % (i+1, entry.title.text)
      if entry.content:
        print '    %s' % (entry.content.text)
      for email in entry.email:
        print '    %s' % (email.address)
      for phone in entry.phone_number:
        print '    %s: %s' %  (phone.rel, phone.text)
      # Show the contact groups that this contact is a member of.
      for group in entry.group_membership_info:
        print '    Member of group: %s' % (group.href)
      # Display extended properties.
      for extended_property in entry.extended_property:
        if extended_property.value:
          value = extended_property.value
        else:
          value = extended_property.GetXmlBlobString()
        print '    Extended Property %s: %s' % (extended_property.name, value)

  def ListAllContacts(self):
    """Retrieves a list of contacts and displays name and primary email."""
    feed = self.gd_client.GetContactsFeed()
    self.PrintFeed(feed)

  def PrintGroupsFeed(self, feed):
    print '\n'
    if not feed.entry:
      print 'No groups in feed.\n'
    for i, entry in enumerate(feed.entry):
      print '\n%s %s' % (i+1, entry.title.text)
      if entry.content:
        print '    %s' % (entry.content.text)
      # Display the group id which can be used to query the contacts feed.
      print '    Group ID: %s' % entry.id.text
      # Display extended properties.
      for extended_property in entry.extended_property:
        if extended_property.value:
          value = extended_property.value
        else:
          value = extended_property.GetXmlBlobString()
        print '    Extended Property %s: %s' % (extended_property.name, value)

  def ListAllGroups(self):
    feed = self.gd_client.GetGroupsFeed()
    self.PrintGroupsFeed(feed)

  def CreateMenu(self):
    """Prompts that enable a user to create a contact."""
    name = raw_input('Enter contact\'s name: ')
    #notes = raw_input('Enter notes for contact: ')
    primary_email = raw_input('Enter primary email address: ')
    phone = raw_input('Enter mobile  number: ')

    new_contact = gdata.contacts.ContactEntry(title=atom.Title(text=name))
    new_contact.content = atom.Content()
    # Create a work email address for the contact and use as primary. 
    new_contact.email.append(gdata.contacts.Email(address=primary_email, 
        primary='true', rel=gdata.contacts.REL_WORK))
    # Create a mobile phone address for the contact and use as primary. 
    new_contact.phone_number.append(gdata.contacts.PhoneNumber(
      primary='true',
      rel=gdata.contacts.REL_MOBILE,text=phone))
    
    entry = self.gd_client.CreateContact(new_contact)

    if entry:
      print 'Creation successful!'
      print 'ID for the new contact:', entry.id.text
    else:
      print 'Upload error.'

  def CreateContact(self,
                    name, email, phone,
                    email_primary='true',
                    phone_rel=gdata.contacts.REL_MOBILE):
    """Create a new contact non-interactively.""""Prompts that enable a user to create a contact."""
    new_contact = gdata.contacts.ContactEntry(title=atom.Title(text=name))
    new_contact.email.append(
      gdata.contacts.Email(address=email, 
                           primary=email_primary))
    new_contact.phone_number.append(gdata.contacts.PhoneNumber(
      primary='true',
      rel=phone_rel,text=phone))
    
    entry = self.gd_client.CreateContact(new_contact)

    if entry:
      print 'Creation successful!'
      print 'ID for the new contact:', entry.id.text
    else:
      print 'Upload error.'

  def QueryMenu(self):
    """Prompts for updated-min query parameters and displays results."""
    updated_min = raw_input(
        'Enter updated min (example: 2007-03-16T00:00:00): ')
    query = gdata.contacts.service.ContactsQuery()
    query.updated_min = updated_min
    feed = self.gd_client.GetContactsFeed(query.ToUri())
    self.PrintFeed(feed)

  def QueryGroupsMenu(self):
    """Prompts for updated-min query parameters and displays results."""
    updated_min = raw_input(
        'Enter updated min (example: 2007-03-16T00:00:00): ')
    query = gdata.service.Query(feed='/m8/feeds/groups/default/full')
    query.updated_min = updated_min
    feed = self.gd_client.GetGroupsFeed(query.ToUri())
    self.PrintGroupsFeed(feed)

  def _SelectContact(self):
    feed = self.gd_client.GetContactsFeed()
    self.PrintFeed(feed)
    selection = 5000
    while selection > len(feed.entry)+1 or selection < 1:
      selection = int(raw_input(
          'Enter the number for the contact you would like to modify: '))
    return feed.entry[selection-1]

  def UpdateContactMenu(self):
    selected_entry = self._SelectContact()
    new_name = raw_input('Enter a new name for the contact: ')
    if not selected_entry.title:
      selected_entry.title = atom.Title()
    selected_entry.title.text = new_name
    self.gd_client.UpdateContact(selected_entry.GetEditLink().href, selected_entry)

  def DeleteContactMenu(self):
    selected_entry = self._SelectContact()
    self.gd_client.DeleteContact(selected_entry.GetEditLink().href)

  def PrintMenu(self):
    """Displays a menu of options for the user to choose from."""
    print ('\nDocument List Sample\n'
           '1) List all of your contacts.\n'
           '2) Create a contact.\n'
           '3) Query contacts on updated time.\n'
           '4) Modify a contact.\n'
           '5) Delete a contact.\n'
           '6) List all of your contact groups.\n'
           '7) Query your groups on updated time.\n'
           '8) Exit.\n')

  def GetMenuChoice(self, max):
    """Retrieves the menu selection from the user.

    Args:
      max: [int] The maximum number of allowed choices (inclusive)

    Returns:
      The integer of the menu item chosen by the user.
    """
    while True:
      choice = raw_input('> ')

      try:
        num = int(choice)
      except ValueError:
        print 'Invalid choice. Please choose a value between 1 and', max
        continue

      if num > max or num < 1:
        print 'Invalid choice. Please choose a value between 1 and', max
      else:
        return num

  def Run(self):
    """Prompts the user to choose funtionality to be demonstrated."""
    try:
      while True:

        self.PrintMenu()

        choice = self.GetMenuChoice(8)

        if choice == 1:
          self.ListAllContacts()
        elif choice == 2:
          self.CreateMenu()
        elif choice == 3:
          self.QueryMenu()
        elif choice == 4:
          self.UpdateContactMenu()
        elif choice == 5:
          self.DeleteContactMenu()
        elif choice == 6:
          self.ListAllGroups()
        elif choice == 7:
          self.QueryGroupsMenu()
        elif choice == 8:
          return

    except KeyboardInterrupt:
      print '\nGoodbye.'
      return

def Shell(user='', pw=''):
  """Starts our Contacts Shell and returns a handle to it."""
  while not user:
    user = raw_input('Please enter your username: ')
  while not pw:
    pw = getpass.getpass()
    if not pw:
      print 'Password cannot be blank.'

  try:
    shell = ContactsShell(user, pw)
  except gdata.service.BadAuthentication:
    print 'Invalid user credentials given.'
    return
  return shell
